@@ -38,16 +38,17 @@ typedef enum {
 	HTML_CHANGE_MIN_WIDTH = 1 << 0,
 	HTML_CHANGE_PREF_WIDTH = 1 << 1,
 	HTML_CHANGE_SIZE = 1 << 2,
-	HTML_CHANGE_WORD_WIDTH = 1 << 3,
+	HTML_CHANGE_RECALC_PI = 1 << 3,
 	HTML_CHANGE_ALL_CALC = ((HTML_CHANGE_SIZE << 1) - 1),
-	HTML_CHANGE_ALL = ((HTML_CHANGE_WORD_WIDTH << 1) - 1)
+	HTML_CHANGE_ALL = ((HTML_CHANGE_RECALC_PI << 1) - 1)
 } HTMLChangeFlags;
 
 typedef enum {
 	HTML_CLEAR_NONE,
 	HTML_CLEAR_LEFT,
 	HTML_CLEAR_RIGHT,
-	HTML_CLEAR_ALL
+	HTML_CLEAR_ALL,
+	HTML_CLEAR_INHERIT
 } HTMLClearType;
 
 /* Paragraph style.  Notice that the `H*' elements must be consecutive.  */
@@ -76,6 +77,7 @@ typedef enum {
 	HTMLHighlightNFColor,
 	HTMLHighlightTextNFColor,
 	HTMLSpellErrorColor,
+	HTMLCiteColor,
 
 	HTMLColors
 } HTMLColorId;
@@ -126,13 +128,16 @@ typedef enum {
 	HTML_LIST_TYPE_ORDERED_LOWER_ROMAN,
 	HTML_LIST_TYPE_ORDERED_UPPER_ROMAN,
 	HTML_LIST_TYPE_MENU,
-	HTML_LIST_TYPE_DIR
+	HTML_LIST_TYPE_DIR,
+	HTML_LIST_TYPE_BLOCKQUOTE,
+	HTML_LIST_TYPE_BLOCKQUOTE_CITE,
+	HTML_LIST_TYPE_GLOSSARY_DL,
+	HTML_LIST_TYPE_GLOSSARY_DD
 } HTMLListType;
 
 typedef enum {
 	HTML_OBJECT_FLAG_NONE = 0,
 	HTML_OBJECT_FLAG_SEPARATOR = 1 << 0,
-	HTML_OBJECT_FLAG_NEWLINE = 1 << 1,
 	HTML_OBJECT_FLAG_SELECTED = 1 << 2,
 	HTML_OBJECT_FLAG_ALLSELECTED = 1 << 3,
 	HTML_OBJECT_FLAG_FIXEDWIDTH = 1 << 4,
@@ -172,7 +177,6 @@ typedef enum {
 	HTML_TYPE_IMAGE,
 	HTML_TYPE_IMAGEINPUT,
 	HTML_TYPE_LINKTEXT,
-	HTML_TYPE_MAP,
 	HTML_TYPE_OBJECT,
 	HTML_TYPE_RADIO,
 	HTML_TYPE_RULE,
@@ -183,7 +187,6 @@ typedef enum {
 	HTML_TYPE_TEXTAREA,
 	HTML_TYPE_TEXTINPUT,
 	HTML_TYPE_TEXTSLAVE,
-	HTML_TYPE_VSPACE,
 	HTML_TYPE_IFRAME,
 	HTML_TYPE_FRAME,
 	HTML_TYPE_FRAMESET,
@@ -215,4 +218,48 @@ typedef enum {
 	HTML_VALIGN_NONE
 } HTMLVAlignType;
 
+typedef enum {
+	DISPLAY_NONE,
+	DISPLAY_INLINE,
+	DISPLAY_LIST_ITEM,
+	DISPLAY_INLINE_BLOCK,
+	DISPLAY_BLOCK,
+	DISPLAY_MARKER,
+	DISPLAY_RUN_IN,
+	DISPLAY_COMPACT,
+	DISPLAY_TABLE_ROW_GROUP,
+	DISPLAY_TABLE_COLUMN,
+	DISPLAY_TABLE_COLUMN_GROUP,
+	DISPLAY_TABLE_HEADER_GROUP,
+	DISPLAY_TABLE_FOOTER_GROUP,
+	DISPLAY_TABLE_ROW,
+	DISPLAY_TABLE_CELL,
+        DISPLAY_TABLE_CAPTION,
+	DISPLAY_INLINE_TABLE,
+	DISPLAY_TABLE,
+	DISPLAY_DOCUMENT
+} HTMLDisplayType;
+
+typedef enum {
+	HTML_WHITE_SPACE_NORMAL,
+	HTML_WHITE_SPACE_PRE,
+	HTML_WHITE_SPACE_NOWRAP,
+	HTML_WHITE_SPACE_PRE_WRAP,
+	HTML_WHITE_SPACE_PRE_LINE,
+	HTML_WHITE_SPACE_INHERIT
+} HTMLWhiteSpaceType;
+
+typedef enum {
+	HTML_LENGTH_TYPE_PIXELS,
+	HTML_LENGTH_TYPE_PERCENT,
+	HTML_LENGTH_TYPE_FRACTION
+} HTMLLengthType;
+
+typedef enum {
+	HTML_BORDER_NONE,
+	HTML_BORDER_SOLID,
+	HTML_BORDER_INSET,
+	HTML_BORDER_OUTSET
+} HTMLBorderStyle;
+
 #endif
