@@ -109,7 +109,6 @@ static int go_position;
 static gboolean use_redirect_filter;
 
 static gboolean slow_loading = FALSE;
-static gboolean exit_when_done = FALSE;
 
 static gint redirect_timerId = 0;
 static gchar *redirect_url = NULL;
@@ -965,11 +964,28 @@ exit_cb (GtkWidget *widget, gpointer data)
 	gtk_main_quit ();
 }
 
-static struct poptOption options[] = {
+/* static struct poptOption options[] = {
   {"slow-loading", '\0', POPT_ARG_NONE, &slow_loading, 0, "Load the document as slowly as possible", NULL},
   {"exit-when-done", '\0', POPT_ARG_NONE, &exit_when_done, 0, "Exit the program as soon as the document is loaded", NULL},
   {NULL}
-};
+  }; */
+
+static gboolean
+motion_notify_event (GtkHTML *html, GdkEventMotion *event, gpointer data)
+{
+	const char *id;
+	GnomeApp *app;
+
+	app = GNOME_APP (data);
+
+	id = gtk_html_get_object_id_at (html, event->x, event->y);
+	if (id)
+		gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), id);
+	else
+		gnome_appbar_set_status (GNOME_APPBAR (app->statusbar), "");
+
+	return FALSE;
+}
 
 gint
 main (gint argc, gchar *argv[])
@@ -977,7 +993,6 @@ main (gint argc, gchar *argv[])
 	GtkWidget *app, *bar;
 	GtkWidget *html_widget;
 	GtkWidget *scrolled_window;
-	GError  *gconf_error  = NULL;
 
 #ifdef MEMDEBUG
 	void *p = malloc (1024);	/* to make linker happy with ccmalloc */
@@ -1059,6 +1074,7 @@ main (gint argc, gchar *argv[])
 	g_signal_connect (html, "redirect", G_CALLBACK (on_redirect), NULL);
 	g_signal_connect (html, "submit", G_CALLBACK (on_submit), NULL);
 	g_signal_connect (html, "object_requested", G_CALLBACK (object_requested_cmd), NULL);
+	g_signal_connect (html, "motion_notify_event", G_CALLBACK (motion_notify_event), app);
 
 #if 0
 	gtk_box_pack_start_defaults (GTK_BOX (hbox), GTK_WIDGET (html));
